@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -1400,6 +1400,7 @@ CodeConvFunc conv_get_code_conv_func(const gchar *src_encoding,
 	case C_ISO_8859_13:
 	case C_ISO_8859_14:
 	case C_ISO_8859_15:
+	case C_ISO_8859_16:
 		break;
 	case C_ISO_2022_JP:
 	case C_ISO_2022_JP_2:
@@ -1634,6 +1635,7 @@ static const struct {
 	{C_WINDOWS_874,		CS_WINDOWS_874},
 	{C_GEORGIAN_PS,		CS_GEORGIAN_PS},
 	{C_TCVN5712_1,		CS_TCVN5712_1},
+	{C_ISO_8859_16,		CS_ISO_8859_16},
 };
 
 static const struct {
@@ -1896,7 +1898,7 @@ CharSet conv_get_locale_charset(void)
 		return cur_charset;
 	}
 
-	if (strcasestr(cur_locale, "UTF-8")) {
+	if (strcasestr(cur_locale, "UTF-8") || strcasestr(cur_locale, "utf8")) {
 		cur_charset = C_UTF_8;
 		return cur_charset;
 	}
@@ -2052,9 +2054,12 @@ const gchar *conv_get_current_locale(void)
 		cur_locale = g_win32_getlocale();
 #else
 		cur_locale = g_getenv("LC_ALL");
-		if (!cur_locale) cur_locale = g_getenv("LC_CTYPE");
-		if (!cur_locale) cur_locale = g_getenv("LANG");
-		if (!cur_locale) cur_locale = setlocale(LC_CTYPE, NULL);
+		if (!cur_locale || *cur_locale == '\0')
+			cur_locale = g_getenv("LC_CTYPE");
+		if (!cur_locale || *cur_locale == '\0')
+			cur_locale = g_getenv("LANG");
+		if (!cur_locale || *cur_locale == '\0')
+			cur_locale = setlocale(LC_CTYPE, NULL);
 #endif /* G_OS_WIN32 */
 
 		debug_print("current locale: %s\n",
@@ -2145,7 +2150,7 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			gint header_len, gboolean addr_field,
 			const gchar *out_encoding)
 {
-	const gchar *cur_encoding;
+	const gchar *src_encoding;
 	gint mimestr_len;
 	gchar *mimesep_enc;
 	gint left;
@@ -2155,22 +2160,24 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 	g_return_if_fail(g_utf8_validate(src, -1, NULL) == TRUE);
 
-	if (MB_CUR_MAX > 1) {
-		use_base64 = TRUE;
-		mimesep_enc = "?B?";
-	} else {
-		use_base64 = FALSE;
-		mimesep_enc = "?Q?";
-	}
-
-	cur_encoding = CS_INTERNAL;
+	src_encoding = CS_INTERNAL;
 	if (!out_encoding)
 		out_encoding = conv_get_outgoing_charset_str();
 	if (!strcmp(out_encoding, CS_US_ASCII))
 		out_encoding = CS_ISO_8859_1;
 
-	mimestr_len = strlen(MIMESEP_BEGIN) + strlen(out_encoding) +
-		strlen(mimesep_enc) + strlen(MIMESEP_END);
+	if (!g_ascii_strncasecmp(out_encoding, "ISO-8859-", 9) ||
+	    !g_ascii_strncasecmp(out_encoding, "KOI8-", 5) ||
+	    !g_ascii_strncasecmp(out_encoding, "Windows-", 8)) {
+		use_base64 = FALSE;
+		mimesep_enc = "?Q?";
+	} else {
+		use_base64 = TRUE;
+		mimesep_enc = "?B?";
+	}
+
+	mimestr_len = strlen(MIMESEP_BEGIN) + strlen(mimesep_enc) +
+		strlen(MIMESEP_END);
 
 	left = MAX_LINELEN - header_len;
 
@@ -2215,9 +2222,11 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 			gchar *out_str;
 			gchar *enc_str;
 			const gchar *p = srcp;
+			const gchar *block_encoding = out_encoding;
 			gint out_str_len;
 			gint out_enc_str_len;
 			gint mime_block_len;
+			gint error = 0;
 			gboolean cont = FALSE;
 
 			while (*p != '\0') {
@@ -2237,12 +2246,13 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				mb_len = g_utf8_skip[*(guchar *)p];
 
 				Xstrndup_a(part_str, srcp, cur_len + mb_len, );
-				out_str = conv_codeset_strdup
-					(part_str, cur_encoding, out_encoding);
-				if (!out_str) {
-					g_warning("conv_encode_header(): code conversion failed\n");
-					conv_unreadable_8bit(part_str);
+				out_str = conv_codeset_strdup_full
+					(part_str, src_encoding, block_encoding,
+					 &error);
+				if (!out_str || error != 0) {
+					g_warning("conv_encode_header(): code conversion failed. Keeping UTF-8.\n");
 					out_str = g_strdup(part_str);
+					block_encoding = CS_UTF_8;
 				}
 				out_str_len = strlen(out_str);
 
@@ -2255,26 +2265,32 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 
 				g_free(out_str);
 
-				if (mimestr_len + out_enc_str_len <= left) {
+				if (mimestr_len + strlen(block_encoding) + out_enc_str_len <= left) {
 					cur_len += mb_len;
 					p += mb_len;
 				} else if (cur_len == 0) {
 					LBREAK_IF_REQUIRED(1, FALSE);
+					if (*p == '"')
+						in_quote ^= TRUE;
 					continue;
 				} else {
 					cont = TRUE;
+					if (*p == '"')
+						in_quote ^= TRUE;
 					break;
 				}
 			}
 
 			if (cur_len > 0) {
 				Xstrndup_a(part_str, srcp, cur_len, );
-				out_str = conv_codeset_strdup
-					(part_str, cur_encoding, out_encoding);
-				if (!out_str) {
+				error = 0;
+				out_str = conv_codeset_strdup_full
+					(part_str, src_encoding, block_encoding,
+					 &error);
+				if (!out_str || error != 0) {
 					g_warning("conv_encode_header(): code conversion failed\n");
-					conv_unreadable_8bit(part_str);
 					out_str = g_strdup(part_str);
+					block_encoding = CS_UTF_8;
 				}
 				out_str_len = strlen(out_str);
 
@@ -2296,10 +2312,13 @@ void conv_encode_header(gchar *dest, gint len, const gchar *src,
 				g_free(out_str);
 
 				/* output MIME-encoded string block */
-				mime_block_len = mimestr_len + strlen(enc_str);
+				mime_block_len = mimestr_len +
+					strlen(block_encoding) +
+					strlen(enc_str);
 				g_snprintf(destp, mime_block_len + 1,
 					   MIMESEP_BEGIN "%s%s%s" MIMESEP_END,
-					   out_encoding, mimesep_enc, enc_str);
+					   block_encoding, mimesep_enc,
+					   enc_str);
 				destp += mime_block_len;
 				srcp += cur_len;
 
