@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -59,6 +59,11 @@ static PrefParam param[] = {
 	{"encoding_method", "0", &prefs_common.encoding_method, P_ENUM},
 	{"mime_filename_encoding_method", "0",
 	 &prefs_common.mime_fencoding_method, P_ENUM},
+	{"check_attach", "FALSE", &prefs_common.check_attach, P_BOOL},
+	{"check_attach_str", NULL, &prefs_common.check_attach_str, P_STRING},
+	{"check_recipients", "FALSE", &prefs_common.check_recipients, P_BOOL},
+	{"check_recp_exclude", NULL, &prefs_common.check_recp_exclude,
+	 P_STRING},
 
 	{"allow_jisx0201_kana", "FALSE", &prefs_common.allow_jisx0201_kana,
 	 P_BOOL},
@@ -138,6 +143,8 @@ static PrefParam param[] = {
 	{"compose_toolbar_setting", NULL,
 	 &prefs_common.compose_toolbar_setting, P_STRING},
 
+	{"layout_type", "0", &prefs_common.layout_type, P_ENUM},
+
 	{"summary_col_show_mark", "TRUE",
 	 &prefs_common.summary_col_visible[S_COL_MARK], P_BOOL},
 	{"summary_col_show_unread", "TRUE",
@@ -248,11 +255,16 @@ static PrefParam param[] = {
 
 	{"summaryview_width", "600", &prefs_common.summaryview_width, P_INT},
 	{"summaryview_height", "180", &prefs_common.summaryview_height, P_INT},
+	{"summaryview_vwidth", "300", &prefs_common.summaryview_vwidth, P_INT},
+	{"summaryview_vheight", "600", &prefs_common.summaryview_vheight,
+	 P_INT},
 
 	{"main_messagewin_x", "256", &prefs_common.main_msgwin_x, P_INT},
 	{"main_messagewin_y", "210", &prefs_common.main_msgwin_y, P_INT},
 	{"messageview_width", "600", &prefs_common.msgview_width, P_INT},
 	{"messageview_height", "300", &prefs_common.msgview_height, P_INT},
+	{"messageview_vwidth", "300", &prefs_common.msgview_vwidth, P_INT},
+	{"messageview_vheight", "600", &prefs_common.msgview_vheight, P_INT},
 	{"messageview_visible", "TRUE", &prefs_common.msgview_visible, P_BOOL},
 
 	{"mainview_x", "64", &prefs_common.mainview_x, P_INT},
@@ -270,10 +282,14 @@ static PrefParam param[] = {
 
 	{"sourcewin_width", "600", &prefs_common.sourcewin_width, P_INT},
 	{"sourcewin_height", "500", &prefs_common.sourcewin_height, P_INT},
+
 	{"compose_x", "32", &prefs_common.compose_x, P_INT},
 	{"compose_y", "32", &prefs_common.compose_y, P_INT},
 	{"compose_width", "600", &prefs_common.compose_width, P_INT},
 	{"compose_height", "560", &prefs_common.compose_height, P_INT},
+
+	{"compose_maximized", "FALSE", &prefs_common.compose_maximized, P_BOOL},
+
 	{"addressbook_x", "32", &prefs_common.addressbook_x, P_INT},
 	{"addressbook_y", "32", &prefs_common.addressbook_y, P_INT},
 	{"addressbook_width", "620", &prefs_common.addressbook_width, P_INT},
@@ -295,6 +311,8 @@ static PrefParam param[] = {
 	 P_BOOL},
 	{"display_header", "TRUE", &prefs_common.display_header, P_BOOL},
 	{"render_html", "TRUE", &prefs_common.render_html, P_BOOL},
+	{"html_only_as_attach", "FALSE", &prefs_common.html_only_as_attach,
+	 P_BOOL},
 	{"line_space", "2", &prefs_common.line_space, P_INT},
 
 	{"textview_cursor_visible", "FALSE",
