@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2007 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -70,6 +70,11 @@ struct _PrefsCommon
 	TransferEncodingMethod encoding_method;
 	MIMEFilenameEncodingMethod mime_fencoding_method;
 
+	gboolean check_attach;
+	gchar *check_attach_str;
+	gboolean check_recipients;
+	gchar *check_recp_exclude;
+
 	gboolean allow_jisx0201_kana;
 
 	/* Compose */
@@ -123,6 +128,8 @@ struct _PrefsCommon
 	gchar *main_toolbar_setting;
 	gchar *compose_toolbar_setting;
 
+	LayoutType layout_type;
+
 	/* Summary columns visibility, position and size */
 	gboolean summary_col_visible[N_SUMMARY_COLS];
 	gint summary_col_pos[N_SUMMARY_COLS];
@@ -144,11 +151,15 @@ struct _PrefsCommon
 
 	gint summaryview_width;
 	gint summaryview_height;
+	gint summaryview_vwidth;
+	gint summaryview_vheight;
 
 	gint main_msgwin_x;
 	gint main_msgwin_y;
 	gint msgview_width;
 	gint msgview_height;
+	gint msgview_vwidth;
+	gint msgview_vheight;
 	gboolean msgview_visible;
 
 	gint mainview_x;
@@ -173,6 +184,8 @@ struct _PrefsCommon
 	gint compose_width;
 	gint compose_height;
 
+	gboolean compose_maximized;
+
 	gint addressbook_x;
 	gint addressbook_y;
 	gint addressbook_width;
@@ -191,6 +204,7 @@ struct _PrefsCommon
 	gboolean display_header;
 	gint line_space;
 	gboolean render_html;
+	gboolean html_only_as_attach;
 	gboolean textview_cursor_visible;
 	gboolean enable_smooth_scroll;
 	gint scroll_step;
