@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2008 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -40,6 +40,8 @@
 #include "utils.h"
 #include "prefs_common.h"
 
+#define MAX_MIME_LEVEL	64
+
 static GHashTable *procmime_get_mime_type_table	(void);
 static GList *procmime_get_mime_type_list	(const gchar *file);
 
@@ -205,7 +207,7 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 	gchar *p;
 	gchar *boundary;
 	gint boundary_len = 0;
-	gchar buf[BUFFSIZE];
+	gchar *buf;
 	glong fpos, prev_fpos;
 
 	g_return_if_fail(mimeinfo != NULL);
@@ -218,15 +220,20 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 	}
 	g_return_if_fail(fp != NULL);
 
+	buf = g_malloc(BUFFSIZE);
+
 	boundary = mimeinfo->boundary;
 
 	if (boundary) {
 		boundary_len = strlen(boundary);
 
 		/* look for first boundary */
-		while ((p = fgets(buf, sizeof(buf), fp)) != NULL)
+		while ((p = fgets(buf, BUFFSIZE, fp)) != NULL)
 			if (IS_BOUNDARY(buf, boundary, boundary_len)) break;
-		if (!p) return;
+		if (!p) {
+			g_free(buf);
+			return;
+		}
 	} else if (mimeinfo->parent && mimeinfo->parent->boundary) {
 		boundary = mimeinfo->parent->boundary;
 		boundary_len = strlen(boundary);
@@ -234,9 +241,12 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 	if ((fpos = ftell(fp)) < 0) {
 		perror("ftell");
+		g_free(buf);
 		return;
 	}
 
+	debug_print("level = %d\n", mimeinfo->level);
+
 	for (;;) {
 		MimeInfo *partinfo;
 		gboolean eom = FALSE;
@@ -256,7 +266,8 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 			mimeinfo->sub = sub = procmime_scan_mime_header(fp);
 			if (!sub) break;
 
-			debug_print("message/rfc822 part found\n");
+			debug_print("message/rfc822 part (content-type: %s)\n",
+				    sub->content_type);
 			sub->level = mimeinfo->level + 1;
 			sub->parent = mimeinfo->parent;
 			sub->main = mimeinfo;
@@ -276,14 +287,14 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 		if (partinfo->mime_type == MIME_MULTIPART ||
 		    partinfo->mime_type == MIME_MESSAGE_RFC822) {
-			if (partinfo->level < 8)
+			if (partinfo->level < MAX_MIME_LEVEL)
 				procmime_scan_multipart_message(partinfo, fp);
 		}
 
 		/* look for next boundary */
 		buf[0] = '\0';
 		is_base64 = partinfo->encoding_type == ENC_BASE64;
-		while ((p = fgets(buf, sizeof(buf), fp)) != NULL) {
+		while ((p = fgets(buf, BUFFSIZE, fp)) != NULL) {
 			if (IS_BOUNDARY(buf, boundary, boundary_len)) {
 				if (buf[2 + boundary_len]     == '-' &&
 				    buf[2 + boundary_len + 1] == '-')
@@ -334,6 +345,8 @@ void procmime_scan_multipart_message(MimeInfo *mimeinfo, FILE *fp)
 
 		if (eom) break;
 	}
+
+	g_free(buf);
 }
 
 void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
@@ -345,15 +358,15 @@ void procmime_scan_encoding(MimeInfo *mimeinfo, const gchar *encoding)
 	g_free(mimeinfo->encoding);
 
 	mimeinfo->encoding = g_strdup(g_strstrip(buf));
-	if (!g_ascii_strcasecmp(buf, "7bit"))
+	if (!g_ascii_strncasecmp(buf, "7bit", 4))
 		mimeinfo->encoding_type = ENC_7BIT;
-	else if (!g_ascii_strcasecmp(buf, "8bit"))
+	else if (!g_ascii_strncasecmp(buf, "8bit", 4))
 		mimeinfo->encoding_type = ENC_8BIT;
-	else if (!g_ascii_strcasecmp(buf, "quoted-printable"))
+	else if (!g_ascii_strncasecmp(buf, "quoted-printable", 16))
 		mimeinfo->encoding_type = ENC_QUOTED_PRINTABLE;
-	else if (!g_ascii_strcasecmp(buf, "base64"))
+	else if (!g_ascii_strncasecmp(buf, "base64", 6))
 		mimeinfo->encoding_type = ENC_BASE64;
-	else if (!g_ascii_strcasecmp(buf, "x-uuencode"))
+	else if (!g_ascii_strncasecmp(buf, "x-uuencode", 10))
 		mimeinfo->encoding_type = ENC_X_UUENCODE;
 	else
 		mimeinfo->encoding_type = ENC_UNKNOWN;
@@ -1658,6 +1671,41 @@ EncodingType procmime_get_encoding_for_text_file(const gchar *file)
 	}
 }
 
+EncodingType procmime_get_encoding_for_str(const gchar *str)
+{
+	const guchar *p;
+	size_t octet_chars = 0;
+	size_t total_len = 0;
+	gfloat octet_percentage;
+
+	total_len = strlen(str);
+
+	for (p = (const guchar *)str; *p != '\0'; ++p) {
+		if (*p & 0x80)
+			++octet_chars;
+	}
+
+	if (total_len > 0)
+		octet_percentage = (gfloat)octet_chars / (gfloat)total_len;
+	else
+		octet_percentage = 0.0;
+
+	debug_print("procmime_get_encoding_for_str(): "
+		    "8bit chars: %d / %d (%f%%)\n", octet_chars, total_len,
+		    100.0 * octet_percentage);
+
+	if (octet_percentage > 0.20) {
+		debug_print("using BASE64\n");
+		return ENC_BASE64;
+	} else if (octet_chars > 0) {
+		debug_print("using quoted-printable\n");
+		return ENC_QUOTED_PRINTABLE;
+	} else {
+		debug_print("using 7bit\n");
+		return ENC_7BIT;
+	}
+}
+
 const gchar *procmime_get_encoding_str(EncodingType encoding)
 {
 	static const gchar *encoding_str[] = {
