@@ -1,6 +1,6 @@
 /*
  * LibSylph -- E-Mail client library
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -398,18 +398,21 @@ gpointer my_memmem(gconstpointer haystack, size_t haystacklen,
 	const gchar *haystack_ = (const gchar *)haystack;
 	const gchar *needle_ = (const gchar *)needle;
 	const gchar *haystack_cur = (const gchar *)haystack;
+	size_t haystack_left = haystacklen;
 
 	if (needlelen == 1)
 		return memchr(haystack_, *needle_, haystacklen);
 
-	while ((haystack_cur = memchr(haystack_cur, *needle_, haystacklen))
+	while ((haystack_cur = memchr(haystack_cur, *needle_, haystack_left))
 	       != NULL) {
 		if (haystacklen - (haystack_cur - haystack_) < needlelen)
 			break;
 		if (memcmp(haystack_cur + 1, needle_ + 1, needlelen - 1) == 0)
 			return (gpointer)haystack_cur;
-		else
+		else {
 			haystack_cur++;
+			haystack_left = haystacklen - (haystack_cur - haystack_);
+		}
 	}
 
 	return NULL;
@@ -489,7 +492,6 @@ gboolean is_next_nonascii(const gchar *s)
 gint get_next_word_len(const gchar *s)
 {
 	const gchar *p = s;
-	gint len = 0;
 	gboolean in_quote = FALSE;
 
 	while (*p != '\0') {
@@ -652,6 +654,36 @@ void extract_parenthesis(gchar *str, gchar op, gchar cl)
 	*destp = '\0';
 }
 
+void extract_parenthesis_with_escape(gchar *str, gchar op, gchar cl)
+{
+	register gchar *srcp, *destp;
+	gint in_brace;
+
+	srcp = destp = str;
+
+	while ((srcp = strchr(srcp, op))) {
+		if (destp > str)
+			*destp++ = ' ';
+		++srcp;
+		in_brace = 1;
+		while (*srcp) {
+			if (*srcp == op)
+				in_brace++;
+			else if (*srcp == cl)
+				in_brace--;
+
+			if (in_brace == 0)
+				break;
+
+			if (*srcp == '\\' && *(srcp + 1) != '\0')
+				++srcp;
+
+			*destp++ = *srcp++;
+		}
+	}
+	*destp = '\0';
+}
+
 void extract_parenthesis_with_skip_quote(gchar *str, gchar quote_chr,
 					 gchar op, gchar cl)
 {
@@ -713,6 +745,25 @@ void extract_quote(gchar *str, gchar quote_chr)
 	}
 }
 
+void extract_quote_with_escape(gchar *str, gchar quote_chr)
+{
+	register gchar *sp, *dp;
+
+	if ((sp = strchr(str, quote_chr))) {
+		dp = sp;
+		++sp;
+		while (*sp) {
+			if (*sp == quote_chr)
+				break;
+			else if (*sp == '\\' && *(sp + 1) != '\0')
+				++sp;
+
+			*dp++ = *sp++;
+		}
+		*dp = '\0';
+	}
+}
+
 void eliminate_address_comment(gchar *str)
 {
 	register gchar *srcp, *destp;
@@ -1020,7 +1071,7 @@ GList *add_history(GList *list, const gchar *str)
 		last = g_list_last(list);
 		if (last) {
 			g_free(last->data);
-			g_list_remove(list, last->data);
+			list = g_list_remove(list, last->data);
 		}
 	}
 
@@ -1355,7 +1406,6 @@ gchar **strsplit_parenthesis(const gchar *str, gchar op, gchar cl,
 	str_array = g_new(gchar*, n);
 
 	i = n - 1;
-
 	str_array[i--] = NULL;
 	for (slist = string_list; slist; slist = slist->next)
 		str_array[i--] = slist->data;
@@ -1415,7 +1465,79 @@ gchar **strsplit_with_quote(const gchar *str, const gchar *delim,
 	str_array = g_new(gchar*, n);
 
 	i = n - 1;
+	str_array[i--] = NULL;
+	for (slist = string_list; slist; slist = slist->next)
+		str_array[i--] = slist->data;
 
+	g_slist_free(string_list);
+
+	return str_array;
+}
+
+gchar **strsplit_csv(const gchar *str, gchar delim, gint max_tokens)
+{
+	GSList *string_list = NULL, *slist;
+	gchar **str_array, *s, *new_str;
+	gchar *tmp, *tmpp, *p;
+	guint i, n = 1, len;
+
+	g_return_val_if_fail(str != NULL, NULL);
+
+	if (max_tokens < 1)
+		max_tokens = G_MAXINT;
+
+	s = strchr_with_skip_quote(str, '"', delim);
+	if (s) {
+		do {
+			len = s - str;
+			tmpp = tmp = g_strndup(str, len);
+
+			if (tmp[0] == '"' && tmp[len - 1] == tmp[0]) {
+				tmp[len - 1] = '\0';
+				++tmpp;
+				p = new_str = g_malloc(len - 1);
+				while (*tmpp) {
+					if (*tmpp == '"' && *(tmpp + 1) == '"')
+						++tmpp;
+					*p++ = *tmpp++;
+				}
+				*p = '\0';
+				g_free(tmp);
+			} else
+				new_str = tmp;
+
+			string_list = g_slist_prepend(string_list, new_str);
+			n++;
+			str = s + 1;
+			s = strchr_with_skip_quote(str, '"', delim);
+		} while (--max_tokens && s);
+	}
+
+	if (*str) {
+		len = strlen(str);
+		tmpp = tmp = g_strdup(str);
+
+		if (tmp[0] == '"' && tmp[len - 1] == tmp[0]) {
+			tmp[len - 1] = '\0';
+			++tmpp;
+			p = new_str = g_malloc(len - 1);
+			while (*tmpp) {
+				if (*tmpp == '"' && *(tmpp + 1) == '"')
+					++tmpp;
+				*p++ = *tmpp++;
+			}
+			*p = '\0';
+			g_free(tmp);
+		} else
+			new_str = tmp;
+
+		string_list = g_slist_prepend(string_list, new_str);
+		n++;
+	}
+
+	str_array = g_new(gchar*, n);
+
+	i = n - 1;
 	str_array[i--] = NULL;
 	for (slist = string_list; slist; slist = slist->next)
 		str_array[i--] = slist->data;
@@ -1724,8 +1846,28 @@ gchar *uriencode_for_filename(const gchar *filename)
 	return enc;
 }
 
+gchar *uriencode_for_mailto(const gchar *mailto)
+{
+	const gchar *p = mailto;
+	gchar *enc, *outp;
+
+	outp = enc = g_malloc(strlen(mailto) * 3 + 1);
+
+	for (p = mailto; *p != '\0'; p++) {
+		if (*p == '+') {
+			*outp++ = '%';
+			get_hex_str(outp, *p);
+			outp += 2;
+		} else
+			*outp++ = *p;
+	}
+
+	*outp = '\0';
+	return enc;
+}
+
 gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
-		     gchar **subject, gchar **body)
+		     gchar **subject, gchar **inreplyto, gchar **body)
 {
 	gchar *tmp_mailto;
 	gchar *p;
@@ -1741,8 +1883,10 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 		p++;
 	}
 
-	if (to && !*to)
-		*to = g_strdup(tmp_mailto);
+	if (to && !*to) {
+		*to = g_malloc(strlen(tmp_mailto) + 1);
+		decode_uri(*to, tmp_mailto);
+	}
 
 	while (p) {
 		gchar *field, *value;
@@ -1765,13 +1909,19 @@ gint scan_mailto_url(const gchar *mailto, gchar **to, gchar **cc, gchar **bcc,
 		if (*value == '\0') continue;
 
 		if (cc && !*cc && !g_ascii_strcasecmp(field, "cc")) {
-			*cc = g_strdup(value);
+			*cc = g_malloc(strlen(value) + 1);
+			decode_uri(*cc, value);
 		} else if (bcc && !*bcc && !g_ascii_strcasecmp(field, "bcc")) {
-			*bcc = g_strdup(value);
+			*bcc = g_malloc(strlen(value) + 1);
+			decode_uri(*bcc, value);
 		} else if (subject && !*subject &&
 			   !g_ascii_strcasecmp(field, "subject")) {
 			*subject = g_malloc(strlen(value) + 1);
 			decode_uri(*subject, value);
+		} else if (inreplyto && !*inreplyto &&
+			   !g_ascii_strcasecmp(field, "in-reply-to")) {
+			*inreplyto = g_malloc(strlen(value) + 1);
+			decode_uri(*inreplyto, value);
 		} else if (body && !*body &&
 			   !g_ascii_strcasecmp(field, "body")) {
 			*body = g_malloc(strlen(value) + 1);
@@ -2100,18 +2250,21 @@ off_t get_left_file_size(FILE *fp)
 
 gboolean file_exist(const gchar *file, gboolean allow_fifo)
 {
-	struct stat s;
-
 	if (file == NULL)
 		return FALSE;
 
-	if (g_stat(file, &s) < 0) {
-		if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
-		return FALSE;
-	}
+	if (allow_fifo) {
+		struct stat s;
 
-	if (S_ISREG(s.st_mode) || (allow_fifo && S_ISFIFO(s.st_mode)))
-		return TRUE;
+		if (g_stat(file, &s) < 0) {
+			if (ENOENT != errno) FILE_OP_ERROR(file, "stat");
+			return FALSE;
+		}
+		if (S_ISREG(s.st_mode) || S_ISFIFO(s.st_mode))
+			return TRUE;
+	} else {
+		return g_file_test(file, G_FILE_TEST_IS_REGULAR);
+	}
 
 	return FALSE;
 }
@@ -2425,8 +2578,15 @@ static gint remove_dir_recursive_real(const gchar *dir)
 	g_free(prev_dir);
 
 	if (g_rmdir(dir) < 0) {
-		FILE_OP_ERROR(dir, "rmdir");
-		return -1;
+		if (ENOTDIR == errno) {
+			if (g_unlink(dir) < 0) {
+				FILE_OP_ERROR(dir, "unlink");
+				return -1;
+			}
+		} else {
+			FILE_OP_ERROR(dir, "rmdir");
+			return -1;
+		}
 	}
 
 	return 0;
@@ -2466,7 +2626,7 @@ leave:
 
 gint rename_force(const gchar *oldpath, const gchar *newpath)
 {
-#ifndef G_OS_UNIX
+#if !defined(G_OS_UNIX) && !GLIB_CHECK_VERSION(2, 9, 1)
 	if (!is_file_entry_exist(oldpath)) {
 		errno = ENOENT;
 		return -1;
@@ -2481,29 +2641,37 @@ gint rename_force(const gchar *oldpath, const gchar *newpath)
 
 gint copy_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 {
-	FILE *src_fp, *dest_fp;
+	gint srcfd, destfd;
 	gint n_read;
-	gchar buf[BUFSIZ];
+	gchar buf[BUFFSIZE];
 	gchar *dest_bak = NULL;
 	gboolean err = FALSE;
 
-	if ((src_fp = g_fopen(src, "rb")) == NULL) {
-		FILE_OP_ERROR(src, "fopen");
+#ifdef G_OS_WIN32
+	if ((srcfd = g_open(src, O_RDONLY | O_BINARY, 0600)) < 0) {
+#else
+	if ((srcfd = g_open(src, O_RDONLY, 0600)) < 0) {
+#endif
+		FILE_OP_ERROR(src, "open");
 		return -1;
 	}
 	if (is_file_exist(dest)) {
 		dest_bak = g_strconcat(dest, ".bak", NULL);
 		if (rename_force(dest, dest_bak) < 0) {
 			FILE_OP_ERROR(dest, "rename");
-			fclose(src_fp);
+			close(srcfd);
 			g_free(dest_bak);
 			return -1;
 		}
 	}
 
-	if ((dest_fp = g_fopen(dest, "wb")) == NULL) {
-		FILE_OP_ERROR(dest, "fopen");
-		fclose(src_fp);
+#ifdef G_OS_WIN32
+	if ((destfd = g_open(dest, O_WRONLY | O_CREAT | O_BINARY, 0600)) < 0) {
+#else
+	if ((destfd = g_open(dest, O_WRONLY | O_CREAT, 0600)) < 0) {
+#endif
+		FILE_OP_ERROR(dest, "open");
+		close(srcfd);
 		if (dest_bak) {
 			if (rename_force(dest_bak, dest) < 0)
 				FILE_OP_ERROR(dest_bak, "rename");
@@ -2512,37 +2680,33 @@ gint copy_file(const gchar *src, const gchar *dest, gboolean keep_backup)
 		return -1;
 	}
 
-	if (change_file_mode_rw(dest_fp, dest) < 0) {
-		FILE_OP_ERROR(dest, "chmod");
-		g_warning(_("can't change file mode\n"));
-	}
+	while ((n_read = read(srcfd, buf, sizeof(buf))) > 0) {
+		gchar *p = buf;
+		const gchar *endp = buf + n_read;
+		gint n_write;
 
-	while ((n_read = fread(buf, sizeof(gchar), sizeof(buf), src_fp)) > 0) {
-		if (n_read < sizeof(buf) && ferror(src_fp))
-			break;
-		if (fwrite(buf, n_read, 1, dest_fp) < 1) {
-			g_warning(_("writing to %s failed.\n"), dest);
-			fclose(dest_fp);
-			fclose(src_fp);
-			g_unlink(dest);
-			if (dest_bak) {
-				if (rename_force(dest_bak, dest) < 0)
-					FILE_OP_ERROR(dest_bak, "rename");
-				g_free(dest_bak);
+		while (p < endp) {
+			if ((n_write = write(destfd, p, endp - p)) < 0) {
+				g_warning(_("writing to %s failed.\n"), dest);
+				close(destfd);
+				close(srcfd);
+				g_unlink(dest);
+				if (dest_bak) {
+					if (rename_force(dest_bak, dest) < 0)
+						FILE_OP_ERROR(dest_bak, "rename");
+					g_free(dest_bak);
+				}
+				return -1;
 			}
-			return -1;
+			p += n_write;
 		}
 	}
 
-	if (ferror(src_fp)) {
-		FILE_OP_ERROR(src, "fread");
-		err = TRUE;
-	}
-	fclose(src_fp);
-	if (fclose(dest_fp) == EOF) {
-		FILE_OP_ERROR(dest, "fclose");
+	if (close(destfd) < 0) {
+		FILE_OP_ERROR(dest, "close");
 		err = TRUE;
 	}
+	close(srcfd);
 
 	if (err) {
 		g_unlink(dest);
@@ -2596,7 +2760,7 @@ gint copy_dir(const gchar *src, const gchar *dest)
 
 gint move_file(const gchar *src, const gchar *dest, gboolean overwrite)
 {
-	if (overwrite == FALSE && is_file_exist(dest)) {
+	if (overwrite == FALSE && is_file_entry_exist(dest)) {
 		g_warning("move_file(): file %s already exists.", dest);
 		return -1;
 	}
@@ -4046,7 +4210,7 @@ void log_print(const gchar *format, ...)
 	g_vsnprintf(buf + TIME_LEN, BUFFSIZE, format, args);
 	va_end(args);
 
-	if (debug_mode) fputs(buf, stdout);
+	if (debug_mode) g_print("%s", buf);
 	log_print_ui_func(buf);
 	if (log_fp) {
 		fputs(buf, log_fp);
