@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999,2000 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2005 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,6 +31,9 @@ gchar *input_dialog_with_invisible	(const gchar	*title,
 gchar *input_dialog_combo		(const gchar	*title,
 					 const gchar	*message,
 					 const gchar	*default_string,
-					 GList		*list);
+					 GList		*list,
+					 gboolean	 case_sensitive);
+gchar *input_dialog_query_password	(const gchar	*server,
+					 const gchar	*user);
 
 #endif /* __INPUTDIALOG_H__ */
