@@ -1,6 +1,6 @@
 /*
  * Sylpheed -- a GTK+ based, lightweight, and fast e-mail client
- * Copyright (C) 1999-2006 Hiroyuki Yamamoto
+ * Copyright (C) 1999-2007 Hiroyuki Yamamoto
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -87,8 +87,16 @@ struct _MainWindow
 	ComboButton *fwd_combo;
 	GtkWidget *delete_btn;
 	GtkWidget *junk_btn;
+	GtkWidget *notjunk_btn;
 	GtkWidget *exec_btn;
 	GtkWidget *next_btn;
+	GtkWidget *prev_btn;
+	GtkWidget *address_btn;
+	GtkWidget *search_btn;
+	GtkWidget *print_btn;
+	GtkWidget *stop_btn;
+	GtkWidget *prefs_common_btn;
+	GtkWidget *prefs_account_btn;
 
 	/* body */
 	GtkWidget *vbox_body;
@@ -143,7 +151,8 @@ MainWindow *main_window_get		(void);
 GtkWidget *main_window_get_folder_window	(MainWindow	*mainwin);
 GtkWidget *main_window_get_message_window	(MainWindow	*mainwin);
 
-void main_window_separation_change	(MainWindow	*mainwin,
+void main_window_change_layout		(MainWindow	*mainwin,
+					 LayoutType	 layout,
 					 SeparateType	 type);
 
 void main_window_toggle_message_view	(MainWindow	*mainwin);
