@@ -50,6 +50,7 @@
 #include "alertpanel.h"
 #include "inputdialog.h"
 #include "manage_window.h"
+#include "printing.h"
 #include "procmsg.h"
 #include "procheader.h"
 #include "procmime.h"
@@ -69,6 +70,9 @@ static void messageview_change_view_type(MessageView		*messageview,
 					 MessageType		 type);
 static void messageview_set_menu_state	(MessageView		*messageview);
 
+static void messageview_set_encoding_menu
+					(MessageView		*messageview);
+
 static gint messageview_delete_cb	(GtkWidget		*widget,
 					 GdkEventAny		*event,
 					 MessageView		*messageview);
@@ -85,6 +89,11 @@ static gboolean key_pressed		(GtkWidget		*widget,
 static void save_as_cb			(gpointer	 data,
 					 guint		 action,
 					 GtkWidget	*widget);
+#if GTK_CHECK_VERSION(2, 10, 0)
+static void page_setup_cb		(gpointer	 data,
+					 guint		 action,
+					 GtkWidget	*widget);
+#endif
 static void print_cb			(gpointer	 data,
 					 guint		 action,
 					 GtkWidget	*widget);
@@ -139,6 +148,10 @@ static GtkItemFactoryEntry msgview_entries[] =
 {
 	{N_("/_File"),			NULL, NULL, 0, "<Branch>"},
 	{N_("/_File/_Save as..."),	NULL, save_as_cb, 0, NULL},
+	{N_("/_File/---"),		NULL, NULL, 0, "<Separator>"},
+#if GTK_CHECK_VERSION(2, 10, 0)
+	{N_("/_File/Page set_up..."),	NULL, page_setup_cb, 0, NULL},
+#endif
 	{N_("/_File/_Print..."),	NULL, print_cb, 0, NULL},
 	{N_("/_File/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_File/_Close"),		NULL, close_cb, 0, NULL},
@@ -181,6 +194,8 @@ static GtkItemFactoryEntry msgview_entries[] =
 	 ENC_ACTION(C_ISO_8859_13)},
 	{N_("/_View/Character _encoding/Baltic (ISO-8859-_4)"),
 	 ENC_ACTION(C_ISO_8859_4)},
+	{N_("/_View/Character _encoding/Baltic (Windows-1257)"),
+	 ENC_ACTION(C_WINDOWS_1257)},
 	ENC_SEPARATOR,
 	{N_("/_View/Character _encoding/Greek (ISO-8859-_7)"),
 	 ENC_ACTION(C_ISO_8859_7)},
@@ -242,7 +257,7 @@ static GtkItemFactoryEntry msgview_entries[] =
 
 	{N_("/_View/---"),		NULL, NULL, 0, "<Separator>"},
 	{N_("/_View/Mess_age source"),	NULL, view_source_cb, 0, NULL},
-	{N_("/_View/Show all _headers"),
+	{N_("/_View/All _headers"),
 					NULL, show_all_header_cb, 0, "<ToggleItem>"},
 
 	{N_("/_Message"),		NULL, NULL, 0, "<Branch>"},
@@ -372,6 +387,7 @@ MessageView *messageview_create_with_new_window(void)
 	GtkWidget *window;
 	GtkWidget *window_vbox;
 	GtkWidget *body_vbox;
+	GtkWidget *vspacer;
 	GtkWidget *menubar;
 	GtkItemFactory *ifactory;
 	GtkWidget *statusbar;
@@ -406,8 +422,11 @@ MessageView *messageview_create_with_new_window(void)
 #endif
 	gtk_box_pack_start(GTK_BOX(window_vbox), menubar, FALSE, TRUE, 0);
 
+	vspacer = gtk_vbox_new(FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(window_vbox), vspacer, FALSE, TRUE,
+			   BORDER_WIDTH);
+
 	body_vbox = gtk_vbox_new(FALSE, BORDER_WIDTH);
-	gtk_container_set_border_width(GTK_CONTAINER(body_vbox), BORDER_WIDTH);
 	gtk_box_pack_start(GTK_BOX(window_vbox), body_vbox, TRUE, TRUE, 0);
 
 	gtk_box_pack_start(GTK_BOX(body_vbox), GTK_WIDGET_PTR(msgview),
@@ -420,8 +439,6 @@ MessageView *messageview_create_with_new_window(void)
 	msgview->statusbar_cid = gtk_statusbar_get_context_id
 		(GTK_STATUSBAR(statusbar), "Message View");
 
-	gtk_widget_show_all(window);
-
 	msgview->new_window = TRUE;
 	msgview->window = window;
 	msgview->window_vbox = window_vbox;
@@ -430,8 +447,12 @@ MessageView *messageview_create_with_new_window(void)
 	msgview->menu_locked = FALSE;
 	msgview->visible = TRUE;
 
+	gtk_widget_show_all(window);
+
 	messageview_init(msgview);
 
+	messageview_set_encoding_menu(msgview);
+
 	ifactory = gtk_item_factory_from_widget(menubar);
 #ifndef G_OS_WIN32
 	action_update_msgview_menu(ifactory, msgview);
@@ -499,7 +520,8 @@ gint messageview_show(MessageView *messageview, MsgInfo *msginfo,
 	textview_set_all_headers(messageview->mimeview->textview, all_headers);
 
 	if (mimeinfo->mime_type != MIME_TEXT &&
-	    mimeinfo->mime_type != MIME_TEXT_HTML) {
+	    (prefs_common.html_only_as_attach ||
+	     mimeinfo->mime_type != MIME_TEXT_HTML)) {
 		messageview_change_view_type(messageview, MVIEW_MIME);
 		mimeview_show_message(messageview->mimeview, mimeinfo, file);
 	} else {
@@ -546,14 +568,57 @@ static void messageview_set_menu_state(MessageView *messageview)
 	GtkWidget *menuitem;
 
 	messageview->menu_locked = TRUE;
+
 	ifactory = gtk_item_factory_from_widget(messageview->menubar);
 	menuitem = gtk_item_factory_get_widget
-		(ifactory, "/View/Show all headers");
+		(ifactory, "/View/All headers");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(menuitem),
 				       messageview->textview->show_all_headers);
+
+	if (messageview->msginfo &&
+	    FOLDER_ITEM_IS_SENT_FOLDER(messageview->msginfo->folder))
+		menu_set_sensitive(ifactory, "/Message/Re-edit", TRUE);
+	else
+		menu_set_sensitive(ifactory, "/Message/Re-edit", FALSE);
+
 	messageview->menu_locked = FALSE;
 }
 
+static void messageview_set_encoding_menu(MessageView *messageview)
+{
+	GtkItemFactoryEntry *entry;
+	GtkItemFactory *ifactory;
+	CharSet encoding;
+	gchar *path, *p, *q;
+	GtkWidget *item;
+
+	encoding = conv_get_charset_from_str(prefs_common.force_charset);
+	ifactory = gtk_item_factory_from_widget(messageview->menubar);
+
+	for (entry = msgview_entries; entry->callback != view_source_cb;
+	     entry++) {
+		if (entry->callback == set_charset_cb &&
+		    (CharSet)entry->callback_action == encoding) {
+			p = q = path = g_strdup(entry->path);
+			while (*p) {
+				if (*p == '_') {
+					if (p[1] == '_') {
+						p++;
+						*q++ = '_';
+					}
+				} else
+					*q++ = *p;
+				p++;
+			}
+			*q = '\0';
+			item = gtk_item_factory_get_item(ifactory, path);
+			gtk_widget_activate(item);
+			g_free(path);
+			break;
+		}
+	}
+}
+
 void messageview_clear(MessageView *messageview)
 {
 	procmsg_msginfo_free(messageview->msginfo);
@@ -744,35 +809,21 @@ static void save_as_cb(gpointer data, guint action, GtkWidget *widget)
 	messageview_save_as(messageview);
 }
 
+#if GTK_CHECK_VERSION(2, 10, 0)
+static void page_setup_cb(gpointer data, guint action, GtkWidget *widget)
+{
+	printing_page_setup_gtk();
+}
+#endif
+
 static void print_cb(gpointer data, guint action, GtkWidget *widget)
 {
 	MessageView *messageview = (MessageView *)data;
-	const gchar *cmdline;
-	gchar *msg;
 
 	if (!messageview->msginfo) return;
 
-	cmdline = prefs_common.print_cmd;
-
-	msg = g_strconcat
-		(_("The message will be printed with the following command:"),
-		 "\n\n", cmdline ? cmdline : _("(Default print command)"),
-		 NULL);
-	if (alertpanel(_("Print"), msg, GTK_STOCK_OK, GTK_STOCK_CANCEL, NULL)
-	    != G_ALERTDEFAULT) {
-		g_free(msg);
-		return;
-	}
-	g_free(msg);
-
-	if (cmdline && str_find_format_times(cmdline, 's') != 1) {
-		alertpanel_error(_("Print command line is invalid:\n`%s'"),
-				 cmdline);
-		return;
-	}
-
-	procmsg_print_message(messageview->msginfo, cmdline,
-			      messageview->textview->show_all_headers);
+	printing_print_message(messageview->msginfo,
+			       messageview->textview->show_all_headers);
 }
 
 static void close_cb(gpointer data, guint action, GtkWidget *widget)
@@ -808,7 +859,9 @@ static void set_charset_cb(gpointer data, guint action, GtkWidget *widget)
 		charset = conv_get_charset_str((CharSet)action);
 		g_free(messageview->forced_charset);
 		messageview->forced_charset = g_strdup(charset);
-		messageview_show(messageview, messageview->msginfo, FALSE);
+		if (messageview->msginfo)
+			messageview_show(messageview, messageview->msginfo,
+					 FALSE);
 	}
 }
 
@@ -866,6 +919,7 @@ static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 	MsgInfo *msginfo;
 	gchar *text = NULL;
 	ComposeMode mode = (ComposeMode)action;
+	gchar *prev_force_charset;
 
 	msginfo = messageview->msginfo;
 	mlist = g_slist_append(NULL, msginfo);
@@ -881,6 +935,9 @@ static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 		mode |= prefs_common.reply_with_quote
 			? COMPOSE_WITH_QUOTE : COMPOSE_WITHOUT_QUOTE;
 
+	prev_force_charset = prefs_common.force_charset;
+	prefs_common.force_charset = messageview->forced_charset;
+
 	switch (COMPOSE_MODE(mode)) {
 	case COMPOSE_REPLY:
 	case COMPOSE_REPLY_TO_SENDER:
@@ -902,6 +959,8 @@ static void reply_cb(gpointer data, guint action, GtkWidget *widget)
 			  mode);
 	}
 
+	prefs_common.force_charset = prev_force_charset;
+
 	/* summary_set_marks_selected(summaryview); */
 	g_free(text);
 	g_slist_free(mlist);
